@@ -1,138 +1,152 @@
 """Arbitrary precision arithmetic."""
 
-__version__ = '5.3.0'
+__version__ = '5.4.0'
 
 try:
-    from ._rithm import (Fraction,
+    from ._rithm import (Endianness,
+                         Fraction,
                          Int)
 except ImportError:
+    from enum import Enum as _Enum
     from math import gcd as _gcd
     from operator import mul as _mul
     from typing import (Any as _Any,
-                        Optional as _Optional,
                         Tuple as _Tuple,
                         Union as _Union,
                         overload as _overload)
 
 
+    class Endianness(_Enum):
+        BIG = 'big'
+        LITTLE = 'little'
+
+        def __repr__(self):
+            return f'rithm.{type(self).__qualname__}.{self.name}'
+
+
     class Int:
-        def bit_length(self) -> 'Int':
+        def bit_length(self):
             return Int(self._value.bit_length())
 
-        def gcd(self, other: 'Int') -> 'Int':
+        def gcd(self, other):
             return Int(_gcd(self._value, other._value))
 
+        def to_bytes(self, endianness):
+            return self._value.to_bytes(_to_bytes_count(self._value),
+                                        endianness.value,
+                                        signed=True)
+
+        @classmethod
+        def from_bytes(cls, value, endianness):
+            return cls(int.from_bytes(value, endianness.value,
+                                      signed=True))
+
         __slots__ = '_value',
 
-        def __new__(cls,
-                    _value: _Union[str, int] = 0,
-                    _base: _Optional[int] = None) -> 'Int':
+        def __new__(cls, _value=0, _base=None):
             self = super().__new__(cls)
-            self._value = (int(_value, _base)
-                           if _base is not None
-                           else int(_value))
+            self._value = (int(_value)
+                           if _base is None
+                           else int(_value, _base))
             return self
 
-        def __abs__(self) -> 'Int':
+        def __abs__(self):
             return Int(abs(self._value))
 
-        def __add__(self, other: 'Int') -> 'Int':
+        def __add__(self, other):
             return (Int(self._value + other._value)
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __and__(self, other: 'Int') -> 'Int':
+        def __and__(self, other):
             return (Int(self._value & other._value)
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __bool__(self) -> bool:
+        def __bool__(self):
             return bool(self._value)
 
-        def __ceil__(self) -> 'Int':
+        def __ceil__(self):
             return self
 
-        def __divmod__(self, other: 'Int') -> _Tuple['Int', 'Int']:
+        def __divmod__(self, other):
             return (tuple(Int(value)
                           for value in divmod(self._value, other._value))
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __eq__(self, other: 'Int') -> bool:
+        def __eq__(self, other):
             return (self._value == other._value
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __float__(self) -> float:
+        def __float__(self):
             return float(self._value)
 
-        def __floor__(self) -> 'Int':
+        def __floor__(self):
             return self
 
-        def __floordiv__(self, other: 'Int') -> 'Int':
+        def __floordiv__(self, other):
             return (Int(self._value // other._value)
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __ge__(self, other: 'Int') -> bool:
+        def __ge__(self, other):
             return (self._value >= other._value
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __getstate__(self) -> int:
+        def __getstate__(self):
             return self._value
 
-        def __gt__(self, other: 'Int') -> bool:
+        def __gt__(self, other):
             return (self._value > other._value
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __hash__(self) -> int:
+        def __hash__(self):
             return hash(self._value)
 
-        def __invert__(self) -> 'Int':
+        def __invert__(self):
             return Int(~self._value)
 
-        def __int__(self) -> int:
+        def __int__(self):
             return self._value
 
-        def __le__(self, other: 'Int') -> bool:
+        def __le__(self, other):
             return (self._value <= other._value
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __lshift__(self, other: 'Int') -> 'Int':
+        def __lshift__(self, other):
             return (Int(self._value << other._value)
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __lt__(self, other: 'Int') -> bool:
+        def __lt__(self, other):
             return (self._value < other._value
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __mod__(self, other: 'Int') -> 'Int':
+        def __mod__(self, other):
             return (Int(self._value % other._value)
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __mul__(self, other: 'Int') -> 'Int':
+        def __mul__(self, other):
             return (Int(self._value * other._value)
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __neg__(self) -> 'Int':
+        def __neg__(self):
             return Int(-self._value)
 
-        def __or__(self, other: 'Int') -> 'Int':
+        def __or__(self, other):
             return (Int(self._value | other._value)
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __pow__(self,
-                    exponent: 'Int',
-                    divisor: _Optional['Int'] = None
-                    ) -> _Union['Fraction', 'Int']:
+        def __pow__(self, exponent, divisor=None):
             return (((Int(self._value ** exponent._value)
                       if exponent >= _ZERO
                       else Fraction(_ONE, self) ** -exponent)
@@ -144,59 +158,63 @@ except ImportError:
                     if isinstance(exponent, Int)
                     else NotImplemented)
 
-        def __repr__(self) -> str:
+        def __repr__(self):
             return f'rithm.Int(\'{self._value}\')'
 
-        def __rshift__(self, other: 'Int') -> 'Int':
+        def __rshift__(self, other):
             return (Int(self._value >> other._value)
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __setstate__(self, state: int) -> None:
+        def __setstate__(self, state: int):
             self._value = state
 
-        def __str__(self) -> str:
+        def __str__(self):
             return str(self._value)
 
-        def __sub__(self, other: 'Int') -> 'Int':
+        def __sub__(self, other):
             return (Int(self._value - other._value)
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __truediv__(self, other: 'Int') -> 'Fraction':
+        def __truediv__(self, other):
             return (Fraction(self, other)
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __trunc__(self) -> 'Int':
+        def __trunc__(self):
             return self
 
-        def __xor__(self, other: 'Int') -> 'Int':
+        def __xor__(self, other):
             return (Int(self._value ^ other._value)
                     if isinstance(other, Int)
                     else NotImplemented)
 
 
-    _ONE = Int('1')
+    def _to_bytes_count(value: int) -> int:
+        return (8 + (value + (value < 0)).bit_length()) // 8
+
+
+    _ONE = Int(1)
     _ZERO = Int()
 
 
     class Fraction:
         @property
-        def denominator(self) -> Int:
+        def denominator(self):
             return self._denominator
 
         @property
-        def numerator(self) -> Int:
+        def numerator(self):
             return self._numerator
 
         __slots__ = '_denominator', '_numerator'
 
         def __new__(cls,
-                    _numerator: _Union[Int, float] = _ZERO,
-                    _denominator: _Optional[Int] = None,
+                    _numerator=_ZERO,
+                    _denominator=None,
                     *,
-                    _normalize: bool = True) -> 'Fraction':
+                    _normalize=True):
             self = super().__new__(cls)
             if _denominator is None:
                 if isinstance(_numerator, float):
@@ -205,19 +223,21 @@ except ImportError:
                     )
                     numerator, denominator = (Int(raw_numerator),
                                               Int(raw_denominator))
-                elif isinstance(_numerator, Int):
-                    numerator, denominator = _numerator, _ONE
+                elif isinstance(_numerator, (Int, int)):
+                    numerator, denominator = Int(_numerator), _ONE
                 else:
                     raise TypeError('First argument should be of '
-                                    f'type {Int} or {float}, '
+                                    f'type {Int}, {int} or {float}, '
                                     f'but found: {type(_numerator)}.')
-            elif isinstance(_denominator, Int):
-                if not isinstance(_numerator, Int):
-                    raise TypeError(f'Numerator should be of type {Int}, '
+            elif isinstance(_denominator, (Int, int)):
+                if not isinstance(_numerator, (Int, int)):
+                    raise TypeError(f'Numerator should be '
+                                    f'of type {Int} or {int}, '
                                     f'but found: {type(_numerator)}.')
-                numerator, denominator = _numerator, _denominator
+                numerator, denominator = Int(_numerator), Int(_denominator)
             else:
-                raise TypeError(f'Denominator should be of type {Int}, '
+                raise TypeError(f'Denominator should be '
+                                f'of type {Int} or {int}, '
                                 f'but found: {type(_denominator)}.')
             if not denominator:
                 raise ZeroDivisionError('Denominator should not be zero.')
@@ -228,7 +248,7 @@ except ImportError:
             self._numerator, self._denominator = numerator, denominator
             return self
 
-        def __abs__(self) -> 'Fraction':
+        def __abs__(self):
             return Fraction(abs(self.numerator), self.denominator,
                             _normalize=False)
 
@@ -237,7 +257,7 @@ except ImportError:
             ...
 
         @_overload
-        def __add__(self, other: _Union['Fraction', Int]) -> 'Fraction':
+        def __add__(self, other: _Union['Fraction', Int]):
             ...
 
         def __add__(self, other: _Union['Fraction', Int, _Any]
@@ -248,10 +268,10 @@ except ImportError:
                           if isinstance(other, Int)
                           else NotImplemented))
 
-        def __bool__(self) -> bool:
+        def __bool__(self):
             return bool(self.numerator)
 
-        def __eq__(self, other: 'Fraction') -> bool:
+        def __eq__(self, other: 'Fraction'):
             return (self.numerator == other.numerator
                     and self.denominator == other.denominator
                     if isinstance(other, Fraction)
@@ -260,7 +280,7 @@ except ImportError:
                           if isinstance(other, Int)
                           else NotImplemented))
 
-        def __float__(self) -> float:
+        def __float__(self):
             return self._numerator._value / self._denominator._value
 
         @_overload
@@ -268,7 +288,7 @@ except ImportError:
             ...
 
         @_overload
-        def __floordiv__(self, other: _Union['Fraction', Int]) -> Int:
+        def __floordiv__(self, other: _Union['Fraction', Int]):
             ...
 
         def __floordiv__(self, other: _Union['Fraction', Int, _Any]
@@ -280,7 +300,7 @@ except ImportError:
                           if isinstance(other, Int)
                           else NotImplemented))
 
-        def __ge__(self, other: 'Fraction') -> bool:
+        def __ge__(self, other: 'Fraction'):
             return (self.numerator * other.denominator
                     >= other.numerator * self.denominator
                     if isinstance(other, Fraction)
@@ -291,7 +311,7 @@ except ImportError:
         def __getstate__(self) -> _Tuple[Int, Int]:
             return self._numerator, self._denominator
 
-        def __gt__(self, other: 'Fraction') -> bool:
+        def __gt__(self, other: 'Fraction'):
             return (self.numerator * other.denominator
                     > other.numerator * self.denominator
                     if isinstance(other, Fraction)
@@ -299,7 +319,7 @@ except ImportError:
                           if isinstance(other, Int)
                           else NotImplemented))
 
-        def __le__(self, other: 'Fraction') -> bool:
+        def __le__(self, other: 'Fraction'):
             return (self.numerator * other.denominator
                     <= other.numerator * self.denominator
                     if isinstance(other, Fraction)
@@ -307,7 +327,7 @@ except ImportError:
                           if isinstance(other, Int)
                           else NotImplemented))
 
-        def __lt__(self, other: 'Fraction') -> bool:
+        def __lt__(self, other: 'Fraction'):
             return (self.numerator * other.denominator
                     < other.numerator * self.denominator
                     if isinstance(other, Fraction)
@@ -320,7 +340,7 @@ except ImportError:
             ...
 
         @_overload
-        def __mod__(self, other: _Union['Fraction', Int]) -> 'Fraction':
+        def __mod__(self, other: _Union['Fraction', Int]):
             ...
 
         def __mod__(self, other: _Union['Fraction', Int, _Any]
@@ -339,7 +359,7 @@ except ImportError:
             ...
 
         @_overload
-        def __mul__(self, other: _Union['Fraction', Int]) -> 'Fraction':
+        def __mul__(self, other: _Union['Fraction', Int]):
             ...
 
         def __mul__(self, other: _Union['Fraction', Int, _Any]
@@ -350,11 +370,11 @@ except ImportError:
                           if isinstance(other, Int)
                           else NotImplemented))
 
-        def __neg__(self) -> 'Fraction':
+        def __neg__(self):
             return Fraction(-self.numerator, self.denominator,
                             _normalize=False)
 
-        def __pow__(self, exponent: 'Int', divisor: None = None) -> 'Fraction':
+        def __pow__(self, exponent, divisor: None = None):
             return ((Fraction(self.numerator ** exponent,
                               self.denominator ** exponent,
                               _normalize=False)
@@ -370,7 +390,7 @@ except ImportError:
                     if isinstance(exponent, Int) and divisor is None
                     else NotImplemented)
 
-        def __repr__(self) -> str:
+        def __repr__(self):
             return f'rithm.Fraction({self.numerator!r}, {self.denominator!r})'
 
         @_overload
@@ -378,7 +398,7 @@ except ImportError:
             ...
 
         @_overload
-        def __radd__(self, other: Int) -> 'Fraction':
+        def __radd__(self, other: Int):
             ...
 
         def __radd__(self, other: _Union[Int, _Any]
@@ -392,7 +412,7 @@ except ImportError:
             ...
 
         @_overload
-        def __rfloordiv__(self, other: Int) -> Int:
+        def __rfloordiv__(self, other: Int):
             ...
 
         def __rfloordiv__(self, other: _Union[Int, _Any]) -> _Union[Int, _Any]:
@@ -405,7 +425,7 @@ except ImportError:
             ...
 
         @_overload
-        def __rmod__(self, other: Int) -> 'Fraction':
+        def __rmod__(self, other: Int):
             ...
 
         def __rmod__(self, other: _Union[Int, _Any]
@@ -420,7 +440,7 @@ except ImportError:
             ...
 
         @_overload
-        def __rmul__(self, other: Int) -> 'Fraction':
+        def __rmul__(self, other: Int):
             ...
 
         def __rmul__(self, other: _Union[Int, _Any]
@@ -434,7 +454,7 @@ except ImportError:
             ...
 
         @_overload
-        def __rsub__(self, other: Int) -> 'Fraction':
+        def __rsub__(self, other: Int):
             ...
 
         def __rsub__(self, other: _Union[Int, _Any]
@@ -450,10 +470,10 @@ except ImportError:
                 else NotImplemented
             )
 
-        def __setstate__(self, state: _Tuple[Int, Int]) -> None:
+        def __setstate__(self, state: _Tuple[Int, Int]):
             self._numerator, self._denominator = state
 
-        def __str__(self) -> str:
+        def __str__(self):
             return (str(self.numerator)
                     if self.denominator == _ONE
                     else f'{self.numerator}/{self.denominator}')
@@ -463,7 +483,7 @@ except ImportError:
             ...
 
         @_overload
-        def __sub__(self, other: _Union['Fraction', Int]) -> 'Fraction':
+        def __sub__(self, other: _Union['Fraction', Int]):
             ...
 
         def __sub__(self, other: _Union['Fraction', Int, _Any]
@@ -492,7 +512,7 @@ except ImportError:
             ...
 
         @_overload
-        def __rtruediv__(self, other: Int) -> 'Fraction':
+        def __rtruediv__(self, other: Int):
             ...
 
         def __rtruediv__(self, other: _Union[Int, _Any]
@@ -506,7 +526,7 @@ except ImportError:
             ...
 
         @_overload
-        def __truediv__(self, other: _Union['Fraction', Int]) -> 'Fraction':
+        def __truediv__(self, other: _Union['Fraction', Int]):
             ...
 
         def __truediv__(self, other: _Union['Fraction', Int, _Any]
@@ -531,7 +551,7 @@ except ImportError:
                       else NotImplemented)
             )
 
-        def _add_fraction(self, other: 'Fraction') -> 'Fraction':
+        def _add_fraction(self, other: 'Fraction'):
             return Fraction(
                     *_normalize_components_moduli(
                             self.numerator * other.denominator
@@ -541,7 +561,7 @@ except ImportError:
                     _normalize=False
             )
 
-        def _add_int(self, other: Int) -> 'Fraction':
+        def _add_int(self, other: Int):
             return Fraction(
                     *_normalize_components_moduli(self.numerator
                                                   + other * self.denominator,
@@ -549,7 +569,7 @@ except ImportError:
                     _normalize=False
             )
 
-        def _mul_by_fraction(self, other: 'Fraction') -> 'Fraction':
+        def _mul_by_fraction(self, other: 'Fraction'):
             numerator, other_denominator = _normalize_components_moduli(
                     self.numerator, other.denominator)
             other_numerator, denominator = _normalize_components_moduli(
@@ -558,13 +578,13 @@ except ImportError:
                             denominator * other_denominator,
                             _normalize=False)
 
-        def _mul_by_int(self, other: Int) -> 'Fraction':
+        def _mul_by_int(self, other: Int):
             other, denominator = _normalize_components_moduli(other,
                                                               self.denominator)
             return Fraction(self.numerator * other, denominator,
                             _normalize=False)
 
-        def _rtruediv_by_int(self, other: Int) -> 'Fraction':
+        def _rtruediv_by_int(self, other: Int):
             other, numerator = _normalize_components_moduli(other,
                                                             self.numerator)
             return Fraction(
@@ -573,7 +593,7 @@ except ImportError:
                     _normalize=False
             )
 
-        def _truediv_by_int(self, other: Int) -> 'Fraction':
+        def _truediv_by_int(self, other: Int):
             numerator, other = _normalize_components_moduli(self.numerator,
                                                             other)
             return Fraction(
