@@ -88,6 +88,7 @@ pub trait DivisibleDigit = BinaryDigit
     + BitLength<Output = usize>
     + DoublePrecision
     + From<u8>
+    + Oppose
     + TryFrom<DoublePrecisionOf<Self>>
     + TryFrom<OppositionOf<DoublePrecisionOf<Self>>>
     + TryFrom<usize>
@@ -368,30 +369,25 @@ pub(crate) fn bitwise_and<Digit: BinaryDigit, const SHIFT: usize>(
     shortest_sign: Sign,
 ) -> (Sign, Vec<Digit>) {
     if longest_sign.is_negative() {
-        longest = complement::<Digit, SHIFT>(&longest);
+        complement_in_place::<Digit, SHIFT>(&mut longest);
     };
     if shortest_sign.is_negative() {
-        shortest = complement::<Digit, SHIFT>(&shortest);
+        complement_in_place::<Digit, SHIFT>(&mut shortest);
     };
-    let result_size = if shortest_sign.is_negative() {
-        longest.len()
-    } else {
-        shortest.len()
+    let mut result = longest;
+    if !shortest_sign.is_negative() {
+        result.truncate(shortest.len());
     };
-    let mut result = Vec::<Digit>::with_capacity(result_size);
     for index in 0..shortest.len() {
-        result.push(longest[index] & shortest[index]);
-    }
-    for index in shortest.len()..result_size {
-        result.push(longest[index]);
+        result[index] &= shortest[index];
     }
     let mut sign = longest_sign & shortest_sign;
     if sign.is_negative() {
         result.push(to_digit_mask::<Digit>(SHIFT));
-        result = complement::<Digit, SHIFT>(&result);
+        complement_in_place::<Digit, SHIFT>(&mut result);
     }
     trim_leading_zeros(&mut result);
-    sign *= (result.len() > 1 || !result[0].is_zero()) as Sign;
+    sign *= to_digits_sign(&result);
     (sign, result)
 }
 
@@ -402,27 +398,22 @@ pub(crate) fn bitwise_or<Digit: BinaryDigit, const SHIFT: usize>(
     shortest_sign: Sign,
 ) -> (Sign, Vec<Digit>) {
     if longest_sign.is_negative() {
-        longest = complement::<Digit, SHIFT>(&longest);
+        complement_in_place::<Digit, SHIFT>(&mut longest);
     };
     if shortest_sign.is_negative() {
-        shortest = complement::<Digit, SHIFT>(&shortest);
+        complement_in_place::<Digit, SHIFT>(&mut shortest);
     };
-    let result_size = if shortest_sign.is_negative() {
-        shortest.len()
-    } else {
-        longest.len()
+    let mut result = longest;
+    if shortest_sign.is_negative() {
+        result.truncate(shortest.len());
     };
-    let mut result = Vec::<Digit>::with_capacity(result_size);
     for index in 0..shortest.len() {
-        result.push(longest[index] | shortest[index]);
-    }
-    for index in shortest.len()..result_size {
-        result.push(longest[index]);
+        result[index] |= shortest[index];
     }
     let sign = longest_sign | shortest_sign;
     if sign.is_negative() {
         result.push(to_digit_mask::<Digit>(SHIFT));
-        result = complement::<Digit, SHIFT>(&result);
+        complement_in_place::<Digit, SHIFT>(&mut result);
     }
     trim_leading_zeros(&mut result);
     (sign, result)
@@ -435,32 +426,35 @@ pub(crate) fn bitwise_xor<Digit: BinaryDigit, const SHIFT: usize>(
     shortest_sign: Sign,
 ) -> (Sign, Vec<Digit>) {
     if longest_sign.is_negative() {
-        longest = complement::<Digit, SHIFT>(&longest);
+        complement_in_place::<Digit, SHIFT>(&mut longest);
     };
     if shortest_sign.is_negative() {
-        shortest = complement::<Digit, SHIFT>(&shortest);
+        complement_in_place::<Digit, SHIFT>(&mut shortest);
     };
-    let mut result = Vec::<Digit>::with_capacity(longest.len());
+    let mut result = longest;
     for index in 0..shortest.len() {
-        result.push(longest[index] ^ shortest[index]);
+        result[index] ^= shortest[index];
     }
     if shortest_sign.is_negative() {
         let digit_mask = to_digit_mask::<Digit>(SHIFT);
-        for index in shortest.len()..longest.len() {
-            result.push(longest[index] ^ digit_mask);
-        }
-    } else {
-        for index in shortest.len()..longest.len() {
-            result.push(longest[index]);
+        for index in shortest.len()..result.len() {
+            result[index] ^= digit_mask;
         }
     };
-    let sign = longest_sign ^ shortest_sign;
-    if sign.is_negative() {
+    let sign_is_negative = longest_sign.is_negative() ^ shortest_sign.is_negative();
+    if sign_is_negative {
         result.push(to_digit_mask::<Digit>(SHIFT));
-        result = complement::<Digit, SHIFT>(&result);
+        complement_in_place::<Digit, SHIFT>(&mut result);
     }
     trim_leading_zeros(&mut result);
-    (sign, result)
+    (
+        if sign_is_negative {
+            -Sign::one()
+        } else {
+            Sign::one()
+        } * to_digits_sign(&result),
+        result,
+    )
 }
 
 pub(crate) fn checked_div_approximation<
@@ -522,7 +516,7 @@ pub(crate) fn checked_div_approximation<
         }
         let quotient_digits_count = dividend_digits_count + shift_digits + 1;
         let mut quotient_data = vec![Digit::zero(); quotient_digits_count];
-        let remainder = shift_digits_left::<Digit, SHIFT>(
+        let remainder = shift_digits_left_in_place::<Digit, SHIFT>(
             dividend_digits,
             ((-shift) as usize) % SHIFT,
             &mut quotient_data[shift_digits..],
@@ -533,7 +527,7 @@ pub(crate) fn checked_div_approximation<
         let mut shift_digits = (shift as usize) / SHIFT;
         let quotient_digits_count = dividend_digits_count - shift_digits;
         let mut quotient_data = vec![Digit::zero(); quotient_digits_count];
-        let remainder = shift_digits_right::<Digit, SHIFT>(
+        let remainder = shift_digits_right_in_place::<Digit, SHIFT>(
             &dividend_digits[shift_digits..],
             (shift as usize) % SHIFT,
             &mut quotient_data,
@@ -561,7 +555,7 @@ pub(crate) fn checked_div_approximation<
         let (next_quotient_digits, remainder) =
             div_rem_two_or_more_digits::<Digit, SHIFT>(&quotient_digits, divisor_digits);
         quotient_digits = next_quotient_digits;
-        if remainder.len() > 1 || !remainder[0].is_zero() {
+        if !to_digits_sign(&remainder).is_zero() {
             inexact = true;
         }
     }
@@ -605,7 +599,7 @@ pub(crate) fn checked_div<Digit: DivisibleDigit, const SHIFT: usize>(
     } else {
         let (digits, _) = div_rem_two_or_more_digits::<Digit, SHIFT>(dividend, divisor);
         Some((
-            dividend_sign * divisor_sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
+            dividend_sign * divisor_sign * to_digits_sign(&digits),
             digits,
         ))
     }
@@ -643,9 +637,9 @@ pub(crate) fn checked_div_euclid<Digit: EuclidDivisibleDigit, const SHIFT: usize
         } else {
             let (digits, remainder) = div_rem_two_or_more_digits::<Digit, SHIFT>(dividend, divisor);
             (
-                dividend_sign * divisor_sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
+                dividend_sign * divisor_sign * to_digits_sign(&digits),
                 digits,
-                remainder.len() > 1 || !remainder[0].is_zero(),
+                !to_digits_sign(&remainder).is_zero(),
             )
         };
         if remainder_is_non_zero
@@ -673,10 +667,7 @@ pub(crate) fn checked_rem<Digit: DivisibleDigit, const SHIFT: usize>(
         Some((dividend_sign * ((!digit.is_zero()) as Sign), vec![digit]))
     } else {
         let (_, digits) = div_rem_two_or_more_digits::<Digit, SHIFT>(dividend, divisor);
-        Some((
-            dividend_sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
-            digits,
-        ))
+        Some((dividend_sign * to_digits_sign(&digits), digits))
     }
 }
 
@@ -708,10 +699,7 @@ pub(crate) fn checked_rem_euclid<Digit: EuclidDivisibleDigit, const SHIFT: usize
             (dividend_sign * ((!digit.is_zero()) as Sign), vec![digit])
         } else {
             let (_, digits) = div_rem_two_or_more_digits::<Digit, SHIFT>(dividend, divisor);
-            (
-                dividend_sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
-                digits,
-            )
+            (dividend_sign * to_digits_sign(&digits), digits)
         };
         if (divisor_sign.is_negative() && sign.is_positive())
             || (divisor_sign.is_positive() && sign.is_negative())
@@ -722,17 +710,15 @@ pub(crate) fn checked_rem_euclid<Digit: EuclidDivisibleDigit, const SHIFT: usize
     }
 }
 
-pub(crate) fn complement<Digit: BinaryDigit, const SHIFT: usize>(digits: &[Digit]) -> Vec<Digit> {
-    let mut result = Vec::<Digit>::with_capacity(digits.len());
+pub(crate) fn complement_in_place<Digit: BinaryDigit, const SHIFT: usize>(digits: &mut [Digit]) {
     let mut accumulator = Digit::one();
     let digit_mask = to_digit_mask::<Digit>(SHIFT);
     for index in 0..digits.len() {
         accumulator += digits[index] ^ digit_mask;
-        result.push(accumulator & digit_mask);
+        digits[index] = accumulator & digit_mask;
         accumulator >>= SHIFT;
     }
     debug_assert!(accumulator.is_zero());
-    result
 }
 
 pub(crate) fn digits_from_finite_positive_improper_float<
@@ -793,9 +779,12 @@ pub(crate) fn div_rem_two_or_more_digits<Digit: DivisibleDigit, const SHIFT: usi
     let mut dividend_normalized = vec![Digit::zero(); dividend_digits_count];
     let mut divisor_normalized = vec![Digit::zero(); divisor_digits_count];
     let shift = SHIFT - divisor[divisor.len() - 1].bit_length();
-    shift_digits_left::<Digit, SHIFT>(divisor, shift, divisor_normalized.as_mut_slice());
-    let accumulator =
-        shift_digits_left::<Digit, SHIFT>(dividend, shift, dividend_normalized.as_mut_slice());
+    shift_digits_left_in_place::<Digit, SHIFT>(divisor, shift, divisor_normalized.as_mut_slice());
+    let accumulator = shift_digits_left_in_place::<Digit, SHIFT>(
+        dividend,
+        shift,
+        dividend_normalized.as_mut_slice(),
+    );
     let last_divisor_digit_normalized = divisor_normalized[divisor_normalized.len() - 1];
     if !accumulator.is_zero()
         || dividend_normalized[dividend_normalized.len() - 1] >= last_divisor_digit_normalized
@@ -876,7 +865,7 @@ pub(crate) fn div_rem_two_or_more_digits<Digit: DivisibleDigit, const SHIFT: usi
     }
     trim_leading_zeros(&mut quotient);
     let mut remainder = divisor_normalized;
-    shift_digits_right::<Digit, SHIFT>(
+    shift_digits_right_in_place::<Digit, SHIFT>(
         &dividend_normalized[..divisor_digits_count],
         shift,
         remainder.as_mut_slice(),
@@ -906,7 +895,7 @@ pub(crate) fn fraction_exponent_digits<
         let shift_digits = (Fraction::MANTISSA_DIGITS + 2 - bits_count) / SHIFT;
         let shift_bits = (Fraction::MANTISSA_DIGITS + 2 - bits_count) % SHIFT;
         let mut result_size = shift_digits;
-        let remainder = shift_digits_left::<Digit, SHIFT>(
+        let remainder = shift_digits_left_in_place::<Digit, SHIFT>(
             digits,
             shift_bits,
             &mut result_digits[result_size..],
@@ -918,7 +907,7 @@ pub(crate) fn fraction_exponent_digits<
     } else {
         let mut shift_digits = (bits_count - Fraction::MANTISSA_DIGITS - 2) / SHIFT;
         let shift_bits = (bits_count - Fraction::MANTISSA_DIGITS - 2) % SHIFT;
-        let remainder = shift_digits_right::<Digit, SHIFT>(
+        let remainder = shift_digits_right_in_place::<Digit, SHIFT>(
             &digits[shift_digits..],
             shift_bits,
             &mut result_digits,
@@ -971,32 +960,6 @@ pub(crate) fn fraction_exponent_digits<
     }
 }
 
-pub(crate) fn left_shift_digits<Digit: LeftShiftableDigit, const SHIFT: usize>(
-    digits: &[Digit],
-    shift_quotient: usize,
-    shift_remainder: Digit,
-) -> Option<Vec<Digit>> {
-    let mut result = Vec::<Digit>::new();
-    result
-        .try_reserve_exact(shift_quotient + ((!shift_remainder.is_zero()) as usize) + digits.len())
-        .ok()?;
-    for _ in 0..shift_quotient {
-        result.push(Digit::zero());
-    }
-    let mut accumulator = DoublePrecisionOf::<Digit>::zero();
-    let digit_mask = to_digit_mask::<DoublePrecisionOf<Digit>>(SHIFT);
-    for &digit in digits {
-        accumulator |= DoublePrecisionOf::<Digit>::from(digit) << shift_remainder;
-        result.push(unsafe { Digit::try_from(accumulator & digit_mask).unwrap_unchecked() });
-        accumulator >>= SHIFT;
-    }
-    if !shift_remainder.is_zero() {
-        result.push(unsafe { Digit::try_from(accumulator).unwrap_unchecked() });
-    }
-    trim_leading_zeros(&mut result);
-    Some(result)
-}
-
 pub(crate) fn multiply_digits<Digit: MultiplicativeDigit, const SHIFT: usize>(
     first: &[Digit],
     second: &[Digit],
@@ -1310,7 +1273,53 @@ where
     result
 }
 
-pub(crate) fn right_shift_digits<Digit: RightShiftableDigit, const SHIFT: usize>(
+pub(crate) fn shift_digits_left<Digit: LeftShiftableDigit, const SHIFT: usize>(
+    digits: &[Digit],
+    shift_quotient: usize,
+    shift_remainder: Digit,
+) -> Option<Vec<Digit>> {
+    let mut result = Vec::<Digit>::new();
+    result
+        .try_reserve_exact(shift_quotient + ((!shift_remainder.is_zero()) as usize) + digits.len())
+        .ok()?;
+    for _ in 0..shift_quotient {
+        result.push(Digit::zero());
+    }
+    let mut accumulator = DoublePrecisionOf::<Digit>::zero();
+    let digit_mask = to_digit_mask::<DoublePrecisionOf<Digit>>(SHIFT);
+    for &digit in digits {
+        accumulator |= DoublePrecisionOf::<Digit>::from(digit) << shift_remainder;
+        result.push(unsafe { Digit::try_from(accumulator & digit_mask).unwrap_unchecked() });
+        accumulator >>= SHIFT;
+    }
+    if !shift_remainder.is_zero() {
+        result.push(unsafe { Digit::try_from(accumulator).unwrap_unchecked() });
+    }
+    trim_leading_zeros(&mut result);
+    Some(result)
+}
+
+fn shift_digits_left_in_place<Digit, const SHIFT: usize>(
+    input: &[Digit],
+    shift: usize,
+    output: &mut [Digit],
+) -> Digit
+where
+    Digit: BinaryDigit + DoublePrecision + TryFrom<DoublePrecisionOf<Digit>>,
+    DoublePrecisionOf<Digit>: BinaryDigit,
+{
+    let mut accumulator: Digit = Digit::zero();
+    let digit_mask = to_digit_mask::<DoublePrecisionOf<Digit>>(SHIFT);
+    for index in 0..input.len() {
+        let step = (DoublePrecisionOf::<Digit>::from(input[index]) << shift)
+            | DoublePrecisionOf::<Digit>::from(accumulator);
+        output[index] = unsafe { Digit::try_from(step & digit_mask).unwrap_unchecked() };
+        accumulator = unsafe { Digit::try_from(step >> SHIFT).unwrap_unchecked() };
+    }
+    accumulator
+}
+
+pub(crate) fn shift_digits_right<Digit: RightShiftableDigit, const SHIFT: usize>(
     digits: &[Digit],
     shift_quotient: usize,
     shift_remainder: Digit,
@@ -1335,27 +1344,7 @@ pub(crate) fn right_shift_digits<Digit: RightShiftableDigit, const SHIFT: usize>
     result
 }
 
-fn shift_digits_left<Digit, const SHIFT: usize>(
-    input: &[Digit],
-    shift: usize,
-    output: &mut [Digit],
-) -> Digit
-where
-    Digit: BinaryDigit + DoublePrecision + TryFrom<DoublePrecisionOf<Digit>>,
-    DoublePrecisionOf<Digit>: BinaryDigit,
-{
-    let mut accumulator: Digit = Digit::zero();
-    let digit_mask = to_digit_mask::<DoublePrecisionOf<Digit>>(SHIFT);
-    for index in 0..input.len() {
-        let step = (DoublePrecisionOf::<Digit>::from(input[index]) << shift)
-            | DoublePrecisionOf::<Digit>::from(accumulator);
-        output[index] = unsafe { Digit::try_from(step & digit_mask).unwrap_unchecked() };
-        accumulator = unsafe { Digit::try_from(step >> SHIFT).unwrap_unchecked() };
-    }
-    accumulator
-}
-
-fn shift_digits_right<Digit, const SHIFT: usize>(
+fn shift_digits_right_in_place<Digit, const SHIFT: usize>(
     input: &[Digit],
     shift: usize,
     output: &mut [Digit],
@@ -1578,6 +1567,11 @@ pub(crate) fn to_digit_mask<Digit: ShiftingLeftMonoid<usize> + SubtractiveMagma
     (Digit::one() << shift) - Digit::one()
 }
 
+#[inline]
+pub(crate) fn to_digits_sign<Digit: Zeroable>(result: &[Digit]) -> Sign {
+    (result.len() > 1 || !result[0].is_zero()) as Sign
+}
+
 pub(crate) fn trim_leading_zeros<Digit>(digits: &mut Vec<Digit>)
 where
     Digit: Clone + Zeroable,
