@@ -10,15 +10,15 @@ use pyo3::basic::CompareOp;
 use pyo3::exceptions::*;
 use pyo3::prelude::{pyclass, pymethods, pymodule, PyModule, PyResult, Python};
 use pyo3::type_object::PyTypeObject;
-use pyo3::types::{PyBytes, PyFloat, PyLong, PyString};
+use pyo3::types::{PyBytes, PyFloat, PyLong, PyString, PyType};
 use pyo3::{AsPyPointer, Py, PyAny, PyErr, PyRef, ToPyObject};
 use pyo3::{IntoPy, PyObject};
 use pyo3_ffi as ffi;
 
 use crate::traits::{
     Abs, BitLength, CheckedDiv, CheckedDivEuclid, CheckedDivRemEuclid, CheckedPow,
-    CheckedPowRemEuclid, CheckedRemEuclid, CheckedShl, CheckedShr, FromStrRadix, Gcd, Oppositive,
-    Pow, Unitary, Zeroable,
+    CheckedPowRemEuclid, CheckedRemEuclid, CheckedShl, CheckedShr, Endianness, FromBytes,
+    FromStrRadix, Gcd, Oppositive, Pow, ToBytes, Unitary, Zeroable,
 };
 
 pub mod big_int;
@@ -34,19 +34,42 @@ type Digit = u32;
 
 const BINARY_SHIFT: usize = (traits::OppositionOf::<Digit>::BITS - 2) as usize;
 const UNDEFINED_DIVISION_ERROR_MESSAGE: &str = "Division by zero is undefined.";
+const PICKLE_SERIALIZATION_ENDIANNESS: Endianness = Endianness::LITTLE;
 
 type _BigInt = big_int::BigInt<Digit, '_', BINARY_SHIFT>;
 type _Fraction = fraction::Fraction<_BigInt>;
 
+#[pyclass(name = "Endianness", module = "rithm")]
+#[derive(Clone)]
+struct PyEndianness(Endianness);
+
+#[pyclass(name = "Fraction", module = "rithm", subclass)]
+#[pyo3(text_signature = "(numerator=None, denominator=None, /)")]
+#[derive(Clone)]
+struct PyFraction(_Fraction);
+
 #[pyclass(name = "Int", module = "rithm", subclass)]
 #[pyo3(text_signature = "(value=None, base=None, /)")]
 #[derive(Clone)]
 struct PyInt(_BigInt);
 
-#[pyclass(name = "Fraction", module = "rithm", subclass)]
-#[pyo3(text_signature = "(numerator=None, denominator=None, /)")]
-#[derive(Clone)]
-struct PyFraction(_Fraction);
+#[pymethods]
+impl PyEndianness {
+    #[classattr]
+    const BIG: PyEndianness = PyEndianness(Endianness::BIG);
+    #[classattr]
+    const LITTLE: PyEndianness = PyEndianness(Endianness::LITTLE);
+
+    fn __repr__(&self) -> String {
+        format!(
+            "rithm.Endianness.{}",
+            match self.0 {
+                Endianness::BIG => "BIG",
+                Endianness::LITTLE => "LITTLE",
+            }
+        )
+    }
+}
 
 #[pymethods]
 impl PyInt {
@@ -61,8 +84,6 @@ impl PyInt {
                         Ok(value) => Ok(PyInt(value)),
                         Err(reason) => Err(PyValueError::new_err(reason.to_string())),
                     }
-                } else if value.is_instance(PyInt::type_object(py))? {
-                    value.extract::<PyInt>()
                 } else if value.is_instance(PyFloat::type_object(py))? {
                     Ok(PyInt(
                         _BigInt::try_from(value.extract::<&PyFloat>()?.value()).map_err(
@@ -77,41 +98,21 @@ impl PyInt {
                         )?,
                     ))
                 } else {
-                    let ptr = value.as_ptr();
-                    let py = value.py();
-                    unsafe {
-                        let value = ffi::PyNumber_Index(ptr);
-                        if value.is_null() {
-                            return Err(PyErr::fetch(py));
-                        }
-                        let bits_count = ffi::_PyLong_NumBits(value);
-                        match bits_count.cmp(&0) {
-                            Ordering::Less => Err(PyErr::fetch(py)),
-                            Ordering::Equal => Ok(PyInt(_BigInt::zero())),
-                            Ordering::Greater => {
-                                let bytes_count = (bits_count as usize) / (u8::BITS as usize) + 1;
-                                let mut buffer = vec![0u8; bytes_count];
-                                if ffi::_PyLong_AsByteArray(
-                                    Py::<PyLong>::from_owned_ptr(py, value).as_ptr()
-                                        as *mut ffi::PyLongObject,
-                                    buffer.as_mut_ptr(),
-                                    buffer.len(),
-                                    1,
-                                    1,
-                                ) < 0
-                                {
-                                    Err(PyErr::fetch(py))
-                                } else {
-                                    Ok(PyInt(_BigInt::from_bytes(buffer)))
-                                }
-                            }
-                        }
-                    }
+                    Ok(PyInt(try_py_integral_to_big_int(value)?))
                 }
             }
         }
     }
 
+    fn to_bytes(&self, py: Python, endianness: PyEndianness) -> PyObject {
+        PyBytes::new(py, &self.0.to_bytes(endianness.0)).to_object(py)
+    }
+
+    #[classmethod]
+    fn from_bytes(_cls: &PyType, mut bytes: Vec<u8>, endianness: PyEndianness) -> PyInt {
+        PyInt(_BigInt::from_bytes(bytes.as_mut_slice(), endianness.0))
+    }
+
     fn bit_length(&self) -> PyInt {
         PyInt(self.0.clone().bit_length())
     }
@@ -170,7 +171,7 @@ impl PyInt {
     }
 
     fn __getstate__(&self, py: Python) -> PyObject {
-        PyBytes::new(py, &self.0.as_bytes()).to_object(py)
+        PyBytes::new(py, &self.0.to_bytes(PICKLE_SERIALIZATION_ENDIANNESS)).to_object(py)
     }
 
     fn __hash__(&self) -> ffi::Py_hash_t {
@@ -271,9 +272,12 @@ impl PyInt {
     }
 
     fn __setstate__(&mut self, py: Python, state: PyObject) -> PyResult<()> {
-        state.extract::<&PyBytes>(py).map(|py_bytes| {
-            self.0 = _BigInt::from_bytes(py_bytes.extract().unwrap());
-        })
+        state
+            .extract::<&PyBytes>(py)
+            .and_then(|py_bytes| py_bytes.extract::<Vec<u8>>())
+            .map(|bytes| {
+                self.0 = _BigInt::from_bytes(&bytes, PICKLE_SERIALIZATION_ENDIANNESS);
+            })
     }
 
     fn __str__(&self) -> String {
@@ -304,7 +308,7 @@ impl PyInt {
 
 #[inline]
 fn big_int_to_py_long(value: &_BigInt) -> PyObject {
-    let buffer = value.as_bytes();
+    let buffer = value.to_bytes(Endianness::LITTLE);
     Python::with_gil(|py| unsafe {
         PyObject::from_owned_ptr(
             py,
@@ -313,14 +317,62 @@ fn big_int_to_py_long(value: &_BigInt) -> PyObject {
     })
 }
 
+#[inline]
+fn try_py_long_to_big_int(value: &PyAny) -> PyResult<_BigInt> {
+    let ptr = value.as_ptr();
+    let py = value.py();
+    unsafe {
+        let value = ffi::PyNumber_Index(ptr);
+        if value.is_null() {
+            return Err(PyErr::fetch(py));
+        }
+        let bits_count = ffi::_PyLong_NumBits(value);
+        match bits_count.cmp(&0) {
+            Ordering::Less => Err(PyErr::fetch(py)),
+            Ordering::Equal => Ok(_BigInt::zero()),
+            Ordering::Greater => {
+                let bytes_count = (bits_count as usize) / (u8::BITS as usize) + 1;
+                let mut buffer = vec![0u8; bytes_count];
+                if ffi::_PyLong_AsByteArray(
+                    Py::<PyLong>::from_owned_ptr(py, value).as_ptr() as *mut ffi::PyLongObject,
+                    buffer.as_mut_ptr(),
+                    buffer.len(),
+                    1,
+                    1,
+                ) < 0
+                {
+                    Err(PyErr::fetch(py))
+                } else {
+                    Ok(_BigInt::from_bytes(
+                        buffer.as_mut_slice(),
+                        Endianness::LITTLE,
+                    ))
+                }
+            }
+        }
+    }
+}
+
+#[inline]
+fn try_py_integral_to_big_int(value: &PyAny) -> PyResult<_BigInt> {
+    if value.is_instance(PyInt::type_object(value.py()))? {
+        Ok(value.extract::<PyInt>()?.0)
+    } else {
+        try_py_long_to_big_int(value)
+    }
+}
+
 #[pymethods]
 impl PyFraction {
     #[new]
-    fn new(_numerator: Option<&PyAny>, _denominator: Option<PyInt>) -> PyResult<Self> {
+    fn new(_numerator: Option<&PyAny>, _denominator: Option<&PyAny>) -> PyResult<Self> {
         match _denominator {
             Some(denominator) => match _numerator {
                 Some(numerator) => {
-                    match _Fraction::new(numerator.extract::<PyInt>()?.0, denominator.0) {
+                    match _Fraction::new(
+                        try_py_integral_to_big_int(numerator)?,
+                        try_py_integral_to_big_int(denominator)?,
+                    ) {
                         Some(value) => Ok(PyFraction(value)),
                         None => Err(PyZeroDivisionError::new_err(
                             UNDEFINED_DIVISION_ERROR_MESSAGE,
@@ -328,15 +380,12 @@ impl PyFraction {
                     }
                 }
                 None => Err(PyTypeError::new_err(
-                    "Numerator should be of type Int, but found None",
+                    "Numerator should be of type `Int` or `int`, but found `None`",
                 )),
             },
             None => Ok(PyFraction(match _numerator {
                 Some(value) => {
-                    let py = value.py();
-                    if value.is_instance(PyInt::type_object(py))? {
-                        _Fraction::new(value.extract::<PyInt>()?.0, _BigInt::one()).unwrap()
-                    } else {
+                    if value.is_instance(PyFloat::type_object(value.py()))? {
                         _Fraction::try_from(value.extract::<f64>()?).map_err(
                             |reason| match reason {
                                 fraction::FromFloatConversionError::NaN => {
@@ -345,6 +394,8 @@ impl PyFraction {
                                 _ => PyOverflowError::new_err(reason.to_string()),
                             },
                         )?
+                    } else {
+                        _Fraction::new(try_py_integral_to_big_int(value)?, _BigInt::one()).unwrap()
                     }
                 }
                 None => _Fraction::zero(),
@@ -674,8 +725,9 @@ fn compare<T: PartialOrd<U>, U>(left: &T, right: &U, op: CompareOp) -> bool {
 fn _rithm(_py: Python, module: &PyModule) -> PyResult<()> {
     module.setattr("__doc__", env!("CARGO_PKG_DESCRIPTION"))?;
     module.setattr("__version__", env!("CARGO_PKG_VERSION"))?;
-    module.add_class::<PyInt>()?;
+    module.add_class::<PyEndianness>()?;
     module.add_class::<PyFraction>()?;
+    module.add_class::<PyInt>()?;
     Ok(())
 }
 
