@@ -12,7 +12,7 @@ non_zero_ints = ints.filter(bool)
 zero_ints = strategies.builds(Int)
 fractions = (strategies.builds(Fraction, ints)
              | strategies.builds(Fraction, ints, non_zero_ints))
-ints_with_builtins = strategies.builds(to_int_with_builtin, integers.map(str))
+ints_with_builtins = strategies.builds(to_int_with_builtin, integers)
 non_zero_ints_with_builtins = ints_with_builtins.filter(all)
 fractions_with_builtins = (strategies.builds(to_fraction_with_builtin,
                                              ints_with_builtins)
@@ -20,6 +20,5 @@ fractions_with_builtins = (strategies.builds(to_fraction_with_builtin,
                                                ints_with_builtins,
                                                non_zero_ints_with_builtins))
 fractions_or_ints_with_builtins = fractions_with_builtins | ints_with_builtins
-small_ints_with_builtins = strategies.builds(
-        to_int_with_builtin, strategies.integers(-100, 100).map(str)
-)
+small_ints_with_builtins = strategies.builds(to_int_with_builtin,
+                                             strategies.integers(-100, 100))
