@@ -6,7 +6,7 @@ from tests.utils import (IntWithBuiltin,
 from . import strategies
 
 
-@given(strategies.ints_with_builtin_ints, strategies.ints_with_builtin_ints)
+@given(strategies.ints_with_builtins, strategies.ints_with_builtins)
 def test_connection_with_builtin(first_with_builtin: IntWithBuiltin,
                                  second_with_builtin: IntWithBuiltin) -> None:
     first, first_builtin = first_with_builtin
